"""
Copyright (c) 2010, Cambridge Silicon Radio Ltd.
Written by Emilio Monti <emilmont@gmail.com>
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this 
list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, 
this list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
3. Neither the name of the Cambridge Silicon Radio Ltd. nor the names of its
contributors may be used to endorse or promote products derived from this
software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""
from MozSymbolsUploader.elf.stream import *
from MozSymbolsUploader.elf.structs import *
from MozSymbolsUploader.elf.exception import ParseError
from io import FileIO
from cStringIO import StringIO
import os 


    
class ELF(ElfStream):
    def __init__(self, initer ):
        if isinstance(initer, str):
            #test if initer is path to a file ...
            if os.path.exists(initer) :
                iobj = FileIO(initer,'rb')
            else : 
                # it should be a binary buffer ..
                iobj = StringIO()
                iobj.write(initer)
                iobj.seek(0,os.SEEK_SET)
        else :
            #it should be an io object (StringIO, FiliIO) 
            #(interface will be great in python)
            iobj = initer
        ElfStream.__init__(self,iobj)
        self.iobj = iobj
        
        # HEADER
        self.header = Header(self)

        # LOAD PROGRAM and SECTION HEADER TABLES
        self.prog_headers = self.load_entries(self.header.ph_offset, self.header.ph_count, ProgramHeader)
        self.sect_headers = self.load_entries(self.header.sh_offset, self.header.sh_count, SectionHeader)
        
        # LOAD SECTION HEADERS STRING TABLE
        strtab = self.sect_headers[self.header.shstrndx]
        self.shstrtab = StringTable(self.io, strtab.offset, strtab.size)
        
        # Create a section dictionary
        self.sect_dict = {}
        for sec in self.sect_headers:
            self.sect_dict[sec.name] = sec
        
    def __del__(self):
        if not self.iobj.closed :
            self.iobj.close()
    
    @staticmethod
    def get_from_file(path):
        return ELF(path)
    
    @staticmethod
    def get_from_file_memory_duplicate(path):
        io = FileIO(path,'rb')
        io2 = StringIO()
        io2.write(io.read())
        io.close()
        io2.seek(0, os.SEEK_SET)
        return ELF(io2)
    
    @staticmethod
    def get_from_memory(bytes):
        io2 = StringIO(bytes)
        io2.write(io.read())
        io.close()
        io2.seek(0, os.SEEK_SET)
        return ELF(io2)
    
